//////////////////////////////////////////////////////
// XmlParseFile.js (1.1)
// Usage:
//   XmlParseFile.js [? | [-vN.M] [<path>]XmlFile [[<path>]XslFile [[<path>]OutFile ]]]
//   Default Parameters:
//     vN.M = v4.0    
//     XmlFile = convert.xml
//     XslFile = convert.xsl
//     OutFile = * [disply only]
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author: Michael Drozdov
// Revision History:
// Ver   Date          Comment
// 1.0   07/31/2002    Initial version started
// 1.1   25/01/2003    Include MSXML-version as option
//////////////////////////////////////////////////////

////////////////////////////////////////////////////
// Constants & global variables
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Source files:
var gsXmlFile = "convert.xml";
var gsXslFile = "convert.xsl";
var gsOutFile = "*";

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Default version:
var gsMsxmlVersion = "4.0";
// Also you can use 3.0|2.6 versions:
//var gsMsxmlVersion = "2.6";
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Referencese to Microsoft.xmldom objects
var goXmlDoc = null;
var goXslDoc = null;
var goXslPrc = null;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Other constants
var gnForWriting = 2;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Code to run:
////////////////////////////////////////////////////
// Get Arguments
try
{
	var oArgObj = WScript.Arguments;
	if (oArgObj.length > 0)
	{
		var nArg = (-1);
		var cCurArg = "";
		var cFirstChr = "";
		for (var i = 0; i < oArgObj.length; i++)
		{
			// Get current arg
			nArg++;
			cCurArg = oArgObj(i);

			// Check help
			if (cCurArg.indexOf("?") != (-1))
			{
				DisplayHelpMessage();
				WScript.Quit(0);
			}

			// Check version
			cFirstChr = cCurArg.charAt(0);
			if (cFirstChr == "-" || cFirstChr == "/" || cFirstChr == "\\")
			{
				cCurArg = cCurArg.substr(1);
				cFirstChr = cCurArg.charAt(0);
				if (cFirstChr == "v" || cFirstChr == "V")
				{
					////////////////////////////////////
					// Reset version
					gsMsxmlVersion = cCurArg.substr(1);
					// Check new version
					if (gsMsxmlVersion.length > 0)
					{
						// Ok!? :-)
						nArg--;
						continue;
					}
				}
				DisplayHelpMessage();
				WScript.Quit(0);
			}
			
			// Get Xml/Xsl/Out file
			switch(nArg)
			{
				case 0:
					gsXmlFile = cCurArg;
					break;
				case 1:
					gsXslFile = cCurArg;
					break;
				case 2:
					gsOutFile = cCurArg;
					break;
				default :
					{
						DisplayHelpMessage();
						WScript.Quit(0);
					}
			}
		}
	}
	oArgObj = null;
}
catch(err)
{
	ShowErr(err, "in translate XmlParseFile.js args");
	WScript.Quit(1);
}
if (typeof(gsOutFile) == "string" && gsOutFile.length == 1 && gsOutFile == "*")
{
	gsOutFile = "";
}

////////////////////////////////////////////////////
// PROGIDs:
var gsPROGIDmsxml = "Msxml2.DOMDocument."+gsMsxmlVersion;
var gsPROGIDmsxsl = "Msxml2.FreeThreadedDOMDocument."+gsMsxmlVersion;
var gsPROGIDmstemp = "Msxml2.XSLTemplate."+gsMsxmlVersion;
var gbMsxml40 = (gsPROGIDmsxml.lastIndexOf("4.0") != (-1));

////////////////////////////////////////////////////
// Create MsXmlDom objects
if (!CreateMsXmlDomObjs())
{
	WScript.Quit(1);
}

////////////////////////////////////////////////////
// Load Xml-file
if (!load(goXmlDoc, gsXmlFile))
{
	WScript.Quit(2);
}

////////////////////////////////////////////////////
// Load Xsl-file
if (!load(goXslDoc, gsXslFile))
{
	WScript.Quit(2);
}

////////////////////////////////////////////////////
// Transformation
var sResult = doTransformation();

////////////////////////////////////////////////////
// Show/Save result
if (typeof(sResult) == "string" && sResult.length > 0 && printResult(sResult))
{
	if (gsOutFile.length > 0)
	{
		WScript.Echo("Ok!");
	}
	WScript.Quit(0);
}
else
{
	WScript.Quit(3);
}
////////////////////////////////////////////////////
// end of run code
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

////////////////////////////////////////////////////
// functions:
//
function DisplayHelpMessage()
{
	WScript.Echo("(ver 1.1) Usage:\n"
		+"XmlParseFile.js [? | [-vN.M] [<path>]XmlFile [[<path>]XslFile [[<path>]OutFile ]]]\n"
		+"  Default Parameters:\n"
		+"    vN.M = v4.0\n"
		+"    XmlFile = convert.xml\n"
		+"    XslFile = convert.xsl\n"
		+"    OutFile = * [disply only]");
}

function CreateMsXmlDomObjs()
{
	var bRetVal = false;
	try
	{
		goXmlDoc = WScript.CreateObject(gsPROGIDmsxml);
		goXslDoc = WScript.CreateObject(gsPROGIDmsxsl);
		goXslPrc = WScript.CreateObject(gsPROGIDmstemp);
		bRetVal = true;
	}
	catch(err)
	{
		ShowErr(err, "in function 'CreateMsXmlDomObjs'");
	}
	return bRetVal;
}

function load(oXmlDom, sSourceFile)
{
    oXmlDom.load(sSourceFile);
    if (oXmlDom.parseError.errorCode != 0)
    {
        WScript.Echo(reportParseError(oXmlDom.parseError)); 
        return false;
    }
    return true;
}

function ShowErr(err, sContext)
{
	WScript.Echo("Error occurred"
		+ "\nCode: " + hex(err.number) 
		+ "\nDescription: " + err.description 
		+ "\n" + sContext);
	return ;
}

function hex(nmb)
{
    if (nmb > 0)
        return nmb.toString(16);
    else
        return (nmb + 0x100000000).toString(16);
}

function reportParseError(error)
{
	var r = "XML Error loading '" + error.url + "'\n"
    	+ "Reason: " + error.reason;
    	if (error.line > 0)
    	{
        	r += "\nat line " + error.line 
        		+ ", character " + error.linepos
        		+ "\n" + error.srcText;
    	}
    	return r;
}

function doTransformation()
{
	var bRetVal = false;
	var sRetVal = "";
	try
	{
	
		// Set Properties if need
		if (gbMsxml40)
		{ 
			goXslDoc.setProperty("SelectionNamespaces",
				"xmlns:xsl='http://www.w3.org/1999/XSL/Transform'");
		}
	
		// Set XSL-document as stylesheet
		goXslPrc.stylesheet = goXslDoc.documentElement;
		
		// Create Processor
		var objXSLTProc = goXslPrc.createProcessor();
		if (objXSLTProc != null)
		{
	
			// Set XML-document
			objXSLTProc.input = goXmlDoc;
			
			// Do transformation
			if (objXSLTProc.transform())
			{
				// Check result
				sRetVal = objXSLTProc.output;
				if (sRetVal.length > 0)
				{
					// Set result
					bRetVal = true;
				}
				else
				{
					WScript.Echo("Empty result of transform"
						+"\nin function 'doTransformation'");
				}
			}
			else
			{
				WScript.Echo("Can not transform\n"+
					"in function 'doTransformation'");
			}
		}
		else
		{
			WScript.Echo("Can not createProcessor\n"+
				"in function 'doTransformation'");
		}
	}
	catch(err)
	{
		ShowErr(err, "in function 'doTransformation'");
	}
	return sRetVal;
}	

function printResult(sResult)
{
	var bRetVal = false;
	if (typeof(sResult) != "string")
	{
		WScript.Echo("Error arg type\n"+
			"in function 'printResult'");
		return bRetVal;
	}
	if (typeof(gsOutFile) == "string" && gsOutFile.length > 0)
	{
		try
		{
			var oFso = WScript.CreateObject("Scripting.FileSystemObject");
			var oTsOut = oFso.OpenTextFile(gsOutFile, gnForWriting, true);
			oTsOut.WriteLine(sResult);
			oTsOut.Close();
			bRetVal = true;
		}
		catch(err)
		{
			ShowErr(err, "in function 'printResult'");
		}
	}
	else
	{
		WScript.Echo(sResult);
		bRetVal = true;
	}
	return bRetVal;
}